import logging
import pkgutil
from argparse import ArgumentParser, RawDescriptionHelpFormatter
from os.path import dirname

from six.moves.urllib.parse import urlparse

from .. import errors
from ..utils import _setup_logging, config, file_or_token, get_ssl_verify_option
from ..utils.api import RepoApi


def create_parser(description=None, epilog=None, version=None):
    parser = ArgumentParser(
        description=description,
        epilog=epilog,
        formatter_class=RawDescriptionHelpFormatter,
    )

    add_parser_default_arguments(parser, version)
    return parser


def add_parser_default_arguments(parser, version):
    output_group = parser.add_argument_group("output")
    output_group.add_argument(
        "--disable-ssl-warnings",
        action="store_true",
        default=False,
        help="Disable SSL warnings (default: %(default)s)",
    )
    output_group.add_argument(
        "-k",
        "--insecure",
        action="store_true",
        dest="insecure",
        help='Allow to perform "insecure" SSL connections and transfers.',
    )
    output_group.add_argument(
        "--show-traceback",
        action="store_true",
        help="Show the full traceback for chalmers user errors (default: %(default)s)",
    )
    output_group.add_argument(
        "-v",
        "--verbose",
        action="store_const",
        help="print debug information on the console",
        dest="log_level",
        default=logging.INFO,
        const=logging.DEBUG,
    )
    output_group.add_argument(
        "-q",
        "--quiet",
        action="store_const",
        help="Only show warnings or errors on the console",
        dest="log_level",
        const=logging.WARNING,
    )
    if version:
        parser.add_argument(
            "-V",
            "--version",
            action="version",
            version="%%(prog)s Command line client (version %s)" % (version,),
        )

    #
    bgroup = parser.add_argument_group("repo-client options")
    bgroup.add_argument(
        "-t",
        "--token",
        type=file_or_token,
        help="Authentication token to use. "
        "May be a token or a path to a file containing a token",
    )
    bgroup.add_argument("-s", "--site", help="select the anaconda-client site to use")

    # add_subparser_modules(parser, sub_command_module, 'conda_server.subcommand')


def get_sub_command_names(module):
    return [
        name
        for _, name, _ in pkgutil.iter_modules([dirname(module.__file__)])
        if not name.startswith("_")
    ]


def get_sub_commands(module):
    names = get_sub_command_names(module)
    this_module = __import__(module.__package__ or module.__name__, fromlist=names)
    for name in names:
        subcmd_module = getattr(this_module, name)
        if hasattr(subcmd_module, "SubCommand"):
            yield getattr(subcmd_module, "SubCommand")


class RepoCommand:
    parser = None
    description = ""
    epilog = ""
    version = ""
    min_api_version = "6.1.0"
    max_api_version = None
    log = logging.getLogger("repo_cli")

    def __init__(self, commands_module, args, metadata=None):
        # (command_module, args, exit, description=__doc__, version=version)
        self._args = args
        self.auth_manager = None
        self._commands_module = commands_module
        self.metadata = metadata or {}
        self.version = self.metadata.get("version")
        self.init_parser()
        self.args = self.parser.parse_args(args)

        _setup_logging(
            self.log,
            log_level=self.args.log_level,
            show_traceback=self.args.show_traceback,
            disable_ssl_warnings=self.args.disable_ssl_warnings,
        )
        self.config = config.get_config(site=self.args.site)
        self.url = self.config.get("url")
        self._access_token = self.args.token or None

        self.verify_ssl = get_ssl_verify_option(
            self.config, self.args.insecure, self.log
        )
        if not self.verify_ssl:
            self.log.warning("SSL verification disabled")

        self.api = RepoApi(
            base_url=self.url, user_token=self._access_token, verify_ssl=self.verify_ssl
        )

    def run(self):
        self.check_token()
        try:
            try:
                if not hasattr(self.args, "main"):
                    self.parser.error(
                        "A sub command must be given. "
                        "To show all available sub commands, run:\n\n\t conda repo -h\n"
                    )
                return self.args.main()

            except errors.Unauthorized:
                if self.args.token:
                    self.log.info(
                        "The provided token does not allow you to perform this operation"
                    )
                    raise SystemExit(1)
                else:
                    self.log.info(
                        "The action you are performing requires authentication, "
                        "please sign in:"
                    )
                    self._access_token = self.auth_manager.login()
                    return self.args.main()

        except errors.ShowHelp:
            self.args.sub_parser.print_help()
            if exit:
                raise SystemExit(1)
            else:
                return 1

    @property
    def site(self):
        return self.args.site

    def check_token(self):
        if not self._access_token:
            # we don't have a valid token... try to get it from local files
            self.api._access_token = self._access_token = config.load_token(self.site)
        return self._access_token

    def init_config(self):
        pass

    def init_parser(self):
        self.parser = create_parser(self.description, self.epilog, self.version)
        self.add_parser_subcommands()

    def add_parser_subcommands(self):
        self._sub_commands = {}
        self.subparsers = self.parser.add_subparsers(help="sub-command help")
        for sub_cmd in get_sub_commands(self._commands_module):
            self.register_sub_command(sub_cmd)

    def register_sub_command(self, sub_command):
        sub_cmd = sub_command(self)
        self._sub_commands[sub_cmd.name] = sub_cmd
        sub_cmd.add_parser(self.subparsers)

        if sub_cmd.manages_auth:
            self.auth_manager = sub_cmd


class SubCommandBase:
    manages_auth = False

    def __init__(self, parent):
        self.parent = parent

    @property
    def args(self):
        return self.parent.args

    @property
    def api(self):
        return self.parent.api

    @property
    def log(self):
        return self.parent.log

    @property
    def access_token(self):
        return self.parent._access_token

    @property
    def verify_ssl(self):
        return self.parent.verify_ssl



import logging

import requests_mock

from repo_cli.commands.config import SubCommand


class TestConfigParent:
    def __init__(self):
        self.log = logging.getLogger(__name__)


def test_set_site_with_no_server(caplog):
    config = SubCommand(TestConfigParent())

    config.set_site({}, "test.com")

    assert len(caplog.records) == 1
    assert caplog.records[0].levelno == logging.ERROR
    assert caplog.records[0].message == "URL is not valid"


def test_set_site_with_server(caplog):
    with requests_mock.Mocker() as mock:
        mock.get("https://www.testurl.com/", status_code=200)
        mock.get(
            "https://www.testurl.com/api/system",
            status_code=200,
            json={"service_name": "repo"},
        )
        config = SubCommand(TestConfigParent())

        config_file = {}
        config.set_site(config_file, "www.testurl.com")

        assert len(caplog.records) == 1
        assert caplog.records[0].levelno == logging.INFO
        assert (
            caplog.records[0].message
            == "Site https://www.testurl.com/api added as testurl.com to configuration"
        )


def test_set_site_with_server_with_invalid_response(caplog):
    with requests_mock.Mocker() as mock:
        mock.get("https://www.testurl.com/", status_code=200)
        mock.get(
            "https://www.testurl.com/api/system",
            status_code=200,
            json={"service_name": "some wrong response"},
        )
        config = SubCommand(TestConfigParent())

        config_file = {}
        config.set_site(config_file, "www.testurl.com")

        assert len(caplog.records) == 1
        assert caplog.records[0].levelno == logging.ERROR
        assert (
            caplog.records[0].message
            == "No Anaconda Server found at https://www.testurl.com/api"
        )


def test_extract_domain():
    config = SubCommand(TestConfigParent())

    assert config.extract_domain("https://www.testurl.com/api") == "testurl.com"
    assert config.extract_domain("https://www.testurl.com") == "testurl.com"
    assert config.extract_domain("https://www.testurl.com/") == "testurl.com"
    assert config.extract_domain("www.testurl.com") == "testurl.com"
    assert config.extract_domain("testurl.com") == "testurl.com"
    assert (
        config.extract_domain("https://www.something.testurl.com/api") == "testurl.com"
    )



"""
`conda repo` configuration

Get, Set, Remove or Show the `conda repo` configuration.

###### Add a `conda repo` site

You can add a site like this:

    conda repo config --set-site "url.of.new_site.com"

    The site will be available under the site name `new_site.com`

You can add a site named **site_name** like this:

    conda repo config --set sites.site_name.url "http://url_of_new_site/api"
    conda repo config --set default_site site_name

    The site will be available under the site name `site_name`.
    Notice that the url explicitly has the `/api` suffix and http / https needs to be specified.

###### `conda repo` sites

`conda repo` sites are a mechanism to allow users to quickly switch
between Anaconda repository instances, from anaconda.org to on-site
[Anaconda Repo](https://server-docs.anaconda.com/en/latest/) installations.

`conda repo` comes one two pre-configured site: `anaconda`. If you
have configured more than one site, you can operate between them
in 2 ways:

  * Invoke the `conda repo` command with the `-s/--site` option
    e.g. to use the alpha testing site:

        conda repo -s mysite whoami

  * Set a site as the default:

        conda repo config --set default_site mysite
        conda repo whoami

  * Show sources:
        conda repo config --show-sources

###### Site Options VS Global Options

All options can be set as global options - affecting all sites,
or site options - affecting only one site

By default options are set globally e.g.:

    conda repo config --set OPTION VALUE

If you want the option to be limited to a single site,
prefix the option with `sites.site_name` e.g.

    conda repo config --set sites.site_name.OPTION VALUE

###### Common `conda repo` configuration options

  * `url`: Set the conda repo api url (default: https://api.anaconda.org)
  * `ssl_verify`: Perform ssl validation on the https requests.
    ssl_verify may be `True`, `False` or a path to a root CA pem file.


###### Other `conda repo` configuration options
  * `auto_register`: Toggle auto_register when doing conda repo upload
  * 'default_site': Default site set to be used by default
  * 'sites': Sites namespace that can be used to add new sites (for
    more information, check the dedicated section above.


###### Toggle auto_register when doing conda repo upload

The default is yes, automatically create a new package when uploading.
If no, then an upload will fail if the package name does not already exist on the server.

    conda repo config --set auto_register yes|no

"""
from __future__ import print_function

import json
from argparse import RawDescriptionHelpFormatter
from urllib.parse import urlparse

import requests

from .. import errors
from ..utils.config import (
    CONFIGURATION_KEYS,
    SEARCH_PATH,
    SYSTEM_CONFIG,
    USER_CONFIG,
    get_config,
    load_config,
    load_file_configs,
    save_config,
)
from ..utils.validators import check_url, check_url_exists
from ..utils.yaml import safe_load, yaml_dump
from .base import SubCommandBase

DEPRECATED = {}


class SubCommand(SubCommandBase):
    name = "config"

    # flake8: noqa: C901
    def main(self):

        config = get_config()

        if self.args.show:
            self.log.info(yaml_dump(config))
            return

        if self.args.show_sources:
            config_files = load_file_configs(SEARCH_PATH)
            for path in config_files:
                self.log.info("==> %s <==", path)
                self.log.info(yaml_dump(config_files[path]))
            return

        if self.args.get:
            if self.args.get in config:
                self.log.info(config[self.args.get])
            else:
                self.log.info("The value of '%s' is not set." % self.args.get)
            return

        if self.args.files:
            from ..utils.config import CONDA_ROOT

            self.log.info("Conda_root: %s" % (CONDA_ROOT,))
            self.log.info("User Config: %s" % USER_CONFIG)
            self.log.info("System Config: %s" % SYSTEM_CONFIG)
            return

        config_file = USER_CONFIG if self.args.user else SYSTEM_CONFIG

        config = load_config(config_file)
        config_system = load_config(SYSTEM_CONFIG)

        for key, value in self.args.set:
            self._validate_url_key(key, value)
            if self.args.user:
                self.recursive_set(config, key, value, self.args.type)
            else:
                self.recursive_set(config_system, key, value, self.args.type)

        for key in self.args.remove:
            try:
                self.recursive_remove(config, key)
            except KeyError:
                self.log.error("Key %s does not exist" % key)

        for site in self.args.set_site:
            if self.args.user:
                self.set_site(config, site)
            else:
                self.set_site(config_system, site)

        if not (self.args.set or self.args.remove or self.args.set_site):
            raise errors.ShowHelp()

        if self.args.user:
            save_config(config, config_file)
        else:
            save_config(config_system, SYSTEM_CONFIG)

    def recursive_remove(self, config_data, key):
        while "." in key:
            if not config_data:
                return
            prefix, key = key.split(".", 1)
            config_data = config_data.get(prefix, {})

        del config_data[key]

    def recursive_set(self, config_data, key, value, type_):
        while "." in key:
            prefix, key = key.split(".", 1)
            config_data = config_data.setdefault(prefix, {})

        if key not in CONFIGURATION_KEYS:
            self.log.warning('"%s" is not a known configuration key', key)

        if key in DEPRECATED.keys():
            message = "{} is deprecated: {}".format(key, DEPRECATED[key])
            self.log.warning(message)

        config_data[key] = type_(value)

    def _validate_url_key(self, key, value):
        if key.endswith(".url"):
            check_url((value))

    def set_site(self, config_data, url):
        if not url.startswith("http"):
            if check_url_exists("https://" + url):
                url = "https://" + url
            elif check_url_exists("http://" + url):
                url = "http://" + url
            else:
                self.log.error("URL is not valid")
                return

        if not url.endswith("/api"):
            url = url.rstrip("/") + "/api"

        check_url(url)
        response = requests.get(url + "/system")

        if response.status_code == 200:
            try:
                json_data = response.json()
            except json.decoder.JSONDecodeError:
                self.log.error("No Anaconda Server found at %s" % url)
                return
            else:
                if json_data.get("service_name") != "repo":
                    self.log.error("No Anaconda Server found at %s" % url)
                    return

            domain_name = self.extract_domain(url)

            if "sites" not in config_data:
                config_data["sites"] = {}

            config_data["sites"][domain_name] = {"url": url}
            config_data["default_site"] = domain_name

            self.log.info("Site %s added as %s to configuration" % (url, domain_name))
        else:
            self.log.error("URL is not valid")

    def extract_domain(self, url):
        parsed_domain = urlparse(url)
        domain = parsed_domain.netloc or parsed_domain.path
        domain_parts = domain.split(".")
        if len(domain_parts) > 2:
            return ".".join(domain_parts[-2:])
        return domain

    def add_parser(self, subparsers):
        description = "Anaconda Repo Client Configuration"
        parser = subparsers.add_parser(
            "config",
            help=description,
            description=description,
            epilog=__doc__,
            formatter_class=RawDescriptionHelpFormatter,
        )

        parser.add_argument(
            "--type",
            default=safe_load,
            help="The type of the values in the set commands. NOTE: This"
            "argument is only used when combined with the `--set`"
            "command and is ignored otherwise.",
        )

        agroup = parser.add_argument_group("actions")

        agroup.add_argument(
            "--set",
            nargs=2,
            action="append",
            default=[],
            help="sets a new variable: name value",
            metavar=("name", "value"),
        )
        agroup.add_argument("--get", metavar="name", help="get value: name")
        agroup.add_argument(
            "--remove", action="append", default=[], help="removes a variable"
        )
        agroup.add_argument(
            "--show", action="store_true", default=False, help="show all variables"
        )
        agroup.add_argument(
            "-f", "--files", action="store_true", help="show the config file names"
        )
        agroup.add_argument(
            "--show-sources",
            action="store_true",
            help="Display all identified config sources",
        )
        agroup.add_argument(
            "--set-site",
            nargs=1,
            action="store",
            default=[],
            help="sets a new site and sets the default_site to this new site",
        )

        lgroup = parser.add_argument_group("location")
        lgroup.add_argument(
            "-u",
            "--user",
            action="store_true",
            dest="user",
            default=True,
            help="set a variable for this user",
        )
        lgroup.add_argument(
            "-s",
            "--system",
            "--site",
            action="store_false",
            dest="user",
            default=False,
            help="set a variable for all users on this machine",
        )

        parser.set_defaults(main=self.main, sub_parser=parser)



from unittest.mock import patch

from requests import Response

from repo_cli.commands.mirror import SubCommand

mirror_resp = {
    "mirror_id": "8c6e75b3-48e5-427d-ab30-2c433ce89e30",
    "mirror_name": "test_mirror",
    "source_root": "https://conda.anaconda.org/waqasanjum",
    "mirror_type": "conda",
    "mirror_mode": "passive",
    "cron": "0 0 * * *",
    "proxy": None,
    "filters": {"only_signed": True},
    "state": "pending",
    "last_run_at": None,
    "created_at": "2023-04-11T15:08:26.646000+00:00",
    "updated_at": "2023-04-11T15:08:26.646000+00:00",
    "sync_state": {
        "logs": {
            "mirror_filtered": {
                "cve": [],
                "date": [],
                "signed": [],
                "license": [],
                "exclude_specs": [],
                "duplicated_legacy_conda": [],
            },
            "channel_filtered": {"license": [], "exclude_specs": []},
        },
        "count": {
            "active": 0,
            "failed": 0,
            "passive": 0,
            "removed": 0,
            "active_planned": 0,
            "mirror_filtered": {
                "cve": 0,
                "date": 0,
                "signed": 0,
                "license": 0,
                "only_specs": 0,
                "exclude_specs": 0,
                "duplicated_legacy_conda": 0,
            },
            "channel_filtered": {"license": 0, "exclude_specs": 0},
        },
        "last_ckey": None,
        "last_exception": None,
        "bytes_transfered": 0,
        "total_steps_count": 8,
        "current_step_number": 0,
        "current_step_percentage": 0,
        "current_step_description": "",
    },
}

mirror_test = {
    "name": "test_mirror",
    "id": "662ac922-4fb7-408a-b8bc-9a00125f14f9",
    "source_root": "https://conda.anaconda.org/jozi22",
    "type": "conda",
    "mode": "passive",
    "state": "running",
    "last_run_at": "2023-04-13T11:28:25.234745+00:00",
    "created_at": "2023-04-06T12:28:48.412000+00:00",
    "updated_at": "2023-04-13T11:28:25.980000+00:00",
    "sync_state": {
        "count": {
            "active": 0,
            "failed": 0,
            "passive": 0,
            "removed": 0,
            "active_planned": 0,
            "mirror_filtered": {
                "cve": 0,
                "date": 0,
                "signed": 0,
                "license": 0,
                "only_specs": 0,
                "exclude_specs": 0,
                "duplicated_legacy_conda": 0,
            },
            "channel_filtered": {"license": 0, "exclude_specs": 0},
        },
        "last_ckey": None,
        "last_exception": None,
        "bytes_transfered": 0,
        "total_steps_count": 8,
        "current_step_number": 3,
        "current_step_percentage": 100,
        "current_step_description": "Filter remote packages",
    },
    "cron": "0 0 1 * *",
    "channel": {"name": "isyed", "privacy": "public"},
}


@patch("repo_cli.commands.base.SubCommandBase")
def test_create_mirror_only_signed(mock_command_base):
    mock_command_base.api.create_mirror.return_value = mirror_resp

    subcommand = SubCommand(mock_command_base)
    subcommand.create_mirror(
        source="https://conda.anaconda.org/jozi22",
        channel="test_channel",
        name="test_create_mirror",
        run_now=True,
        type_="conda",
        mode="passive",
        proxy=None,
        cron=None,
        filter_licenses=None,
        filter_subdirs=None,
        filter_projects=None,
        filter_only_specs=None,
        filter_exclude_specs=None,
        filter_include_specs=None,
        cve_score=None,
        exclude_uncurrated_cve_packages=None,
        filter_only_signed=True,
        filter_date_from=None,
        filter_date_to=None,
    )
    mock_command_base.api.create_mirror.assert_called_once()
    mock_command_base.api.create_mirror.assert_called_with(
        "test_channel",
        "https://conda.anaconda.org/jozi22",
        "test_create_mirror",
        "passive",
        {"only_signed": True},
        "conda",
        None,
        True,
        None,
    )


@patch("repo_cli.commands.mirror.SubCommandBase")
@patch("repo_cli.commands.mirror.SubCommand._get_by_name")
def test_create_mirror_with_existing_mirror(mock_command, mock_command_base):
    mock_command.return_value = "mirror"

    subcommand = SubCommand(mock_command_base)
    subcommand.create_mirror(
        source="https://conda.anaconda.org/jozi22",
        channel="test_channel",
        name="test_create_mirror",
        run_now=True,
        type_="conda",
        mode="passive",
        proxy=None,
        cron=None,
        filter_licenses=None,
        filter_subdirs=None,
        filter_projects=None,
        filter_only_specs=None,
        filter_exclude_specs=None,
        filter_include_specs=None,
        cve_score=None,
        exclude_uncurrated_cve_packages=None,
        filter_only_signed=False,
        filter_date_from=None,
        filter_date_to=None,
    )
    mock_command.assert_called_once()
    mock_command_base.api.create_mirror.assert_not_called()


@patch("repo_cli.commands.base.SubCommandBase")
@patch("repo_cli.commands.mirror.SubCommand._get_by_name")
def test_update_mirror_only_signed(mock_command, mock_command_base):
    mock_command.return_value = {
        "mirror_id": "8c6e75b3-48e5-427d-ab30-2c433ce89e30",
        "mirror_name": "test_create_mirror",
        "source_root": "https://conda.anaconda.org/jozi22",
        "mirror_type": "conda",
        "mirror_mode": "passive",
        "cron": "0 0 * * *",
        "proxy": None,
        "filters": {"only_signed": False},
    }
    mock_command_base.api.update_mirror.return_value = mirror_resp

    subcommand = SubCommand(mock_command_base)
    subcommand.update_mirror(
        source="https://conda.anaconda.org/jozi22",
        channel="test_channel",
        name="test_create_mirror",
        new_name=None,
        run_now=True,
        type_="conda",
        mode="passive",
        proxy=None,
        cron=None,
        filter_licenses=None,
        filter_subdirs=None,
        filter_projects=None,
        filter_only_specs=None,
        filter_exclude_specs=None,
        filter_include_specs=None,
        cve_score=None,
        exclude_uncurrated_cve_packages=None,
        filter_only_signed=True,
        filter_date_from=None,
        filter_date_to=None,
    )
    mock_command_base.api.update_mirror.assert_called_once()
    mock_command_base.api.update_mirror.assert_called_with(
        "8c6e75b3-48e5-427d-ab30-2c433ce89e30",
        "test_channel",
        "https://conda.anaconda.org/jozi22",
        "test_create_mirror",
        "passive",
        {"only_signed": True},
        "conda",
        "0 0 * * *",
        True,
        None,
    )


@patch("repo_cli.commands.mirror.SubCommandBase")
@patch("repo_cli.commands.mirror.SubCommand._get_by_name")
def test_update_mirror_with_no_existing_mirror(mock_command, mock_command_base):
    mock_command.return_value = None

    subcommand = SubCommand(mock_command_base)
    subcommand.update_mirror(
        source="https://conda.anaconda.org/jozi22",
        channel="test_channel",
        name="test_create_mirror",
        new_name=None,
        run_now=True,
        type_="conda",
        mode="passive",
        proxy=None,
        cron=None,
        filter_licenses=None,
        filter_subdirs=None,
        filter_projects=None,
        filter_only_specs=None,
        filter_exclude_specs=None,
        filter_include_specs=None,
        cve_score=None,
        exclude_uncurrated_cve_packages=None,
        filter_only_signed=False,
        filter_date_from=None,
        filter_date_to=None,
    )
    mock_command.assert_called_once()
    mock_command_base.api.update_mirror.assert_not_called()


@patch("repo_cli.commands.base.SubCommandBase")
def test_stop_running_mirror(mock_command_base):
    mirror_data = {"items": [mirror_test]}
    mock_command_base.api.get_all_mirrors.return_value = mirror_data
    mock_command_base.api.stop_mirror.return_value = Response.ok

    subcommand = SubCommand(mock_command_base)
    mirror_data["items"][0]["state"] = "running"
    subcommand.stop(mirror_name="test_mirror")
    mock_command_base.api.get_all_mirrors.asser_called_once()
    mock_command_base.api.stop_mirror.assert_called_once()


@patch("repo_cli.commands.base.SubCommandBase")
def test_stop_pending_mirror(mock_command_base):
    mirror_data = {"items": [mirror_test]}
    mock_command_base.api.get_all_mirrors.return_value = mirror_data
    mock_command_base.api.stop_mirror.return_value = Response.ok

    subcommand = SubCommand(mock_command_base)

    mirror_data["items"][0]["state"] = "pending"
    subcommand.stop(mirror_name="test_mirror")
    mock_command_base.api.get_all_mirrors.asser_called_once()
    mock_command_base.api.stop_mirror.assert_called_once()


@patch("repo_cli.commands.base.SubCommandBase")
def test_stop_completed_mirror(mock_command_base):
    mirror_data = {"items": [mirror_test]}
    mirror_data["items"][0]["state"] = "completed"
    mock_command_base.api.get_all_mirrors.return_value = mirror_data
    mock_command_base.api.stop_mirror.return_value = Response.ok

    subcommand = SubCommand(mock_command_base)
    subcommand.stop(mirror_name="test_mirror")
    mock_command_base.api.get_all_mirrors.asser_called_once()
    mock_command_base.api.stop_mirror.assert_not_called()


@patch("repo_cli.commands.base.SubCommandBase")
def test_stop_stopped_mirror(mock_command_base):
    mirror_data = {"items": [mirror_test]}
    mirror_data["items"][0]["state"] = "stopped"
    mock_command_base.api.get_all_mirrors.return_value = mirror_data
    mock_command_base.api.stop_mirror.return_value = Response.ok

    subcommand = SubCommand(mock_command_base)
    subcommand.stop(mirror_name="test_mirror")
    mock_command_base.api.get_all_mirrors.asser_called_once()
    mock_command_base.api.stop_mirror.assert_not_called()


@patch("repo_cli.commands.base.SubCommandBase")
def test_restart_mirror(mock_command_base):
    mirror_data = {"items": [mirror_test]}
    mock_command_base.api.get_all_mirrors.return_value = mirror_data
    mock_command_base.api.update_mirror.return_value = mirror_test

    subcommand = SubCommand(mock_command_base)
    subcommand.restart(name="test_mirror")
    mock_command_base.api.get_all_mirrors.asser_called_once()
    mock_command_base.api.update_mirror.assert_called_once()


@patch("repo_cli.commands.base.SubCommandBase")
def test_restart_not_existing_mirror(mock_command_base):
    mirror_data = {"items": [mirror_test]}
    mock_command_base.api.get_all_mirrors.return_value = mirror_data
    mock_command_base.api.update_mirror.return_value = mirror_test

    subcommand = SubCommand(mock_command_base)
    subcommand.restart(name="test_mirror_not")
    mock_command_base.api.get_all_mirrors.asser_called_once()
    mock_command_base.api.update_mirror.assert_not_called()
